/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */
#ifndef _SFXDOCTDLG_HXX
#define _SFXDOCTDLG_HXX

#include <comphelper/string.hxx>
#include <sfx2/doctempl.hxx>
#include <vcl/lstbox.hxx>
#include <vcl/edit.hxx>
#include <vcl/fixed.hxx>
#include <vcl/button.hxx>
#include <vcl/dialog.hxx>

class SfxModalDefParentHelper;

class SfxDocumentTemplateDlg : public ModalDialog
{
private:

    FixedLine       aEditFL;
    Edit            aNameEd;
    FixedLine       aTemplateFL;
    FixedText       aRegionFt;
    ListBox         aRegionLb;
    FixedText       aTemplateFt;
    ListBox         aTemplateLb;

    OKButton        aOkBt;
    CancelButton    aCancelBt;
    HelpButton      aHelpBt;
    PushButton      aEditBt;
    PushButton      aOrganizeBt;

    SfxDocumentTemplates *pTemplates;

    SfxModalDefParentHelper* pHelper;

    void Init();

    DECL_LINK( OrganizeHdl, Button * );
    DECL_LINK( OkHdl, Control * );
    DECL_LINK( RegionSelect, ListBox * );
    DECL_LINK( TemplateSelect, ListBox * );
    DECL_LINK( NameModify, Edit * );
    DECL_LINK( EditHdl, Button * );

public:
    SfxDocumentTemplateDlg(Window * pParent, SfxDocumentTemplates* pTempl );
    ~SfxDocumentTemplateDlg();

    String GetTemplateName() const
    { return comphelper::string::stripStart(aNameEd.GetText(), ' '); }
    sal_uInt16 GetRegion() const { return aRegionLb.GetSelectEntryPos(); }
    String GetRegionName() const { return aRegionLb.GetSelectEntry(); }
};

#endif

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
